import { world, system } from '@minecraft/server'

const woodTypes = [
  'oak','spruce','spicewood','cinder','pale','mangrove',
  'dark_oak','jungle','acacia','crimson','warped','cherry','birch'
]
const directions = ['north','south','west','east']

function getWoodType(block) {
  return woodTypes.find(w => block.typeId.endsWith(`_${w}`))
}

function toggleLamp(block, turnOn, player, visited = new Set()) {
  if (!block.typeId.startsWith('ff:lamp_on_') && !block.typeId.startsWith('ff:lamp_off_')) return
  const wood = getWoodType(block)
  if (!wood) return

  const key = `${block.location.x},${block.location.y},${block.location.z}`
  if (visited.has(key)) return
  visited.add(key)

  const { x, y, z } = block.location
  const cx = x + 0.5, cy = y, cz = z + 0.5
  const dir    = block.permutation.getState("minecraft:cardinal_direction")
  const color  = block.permutation.getState("ef:colors")
  const topBit = block.permutation.getState("ff:top_bit")
  if (!directions.includes(dir) || color < 0 || color > 15) return

  if (turnOn && !topBit) {
    block.dimension.runCommand(
      `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
    )
  }
  if (!turnOn && !topBit) {
    block.dimension.runCommand(
      `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
    )
    block.dimension.runCommand(
      `summon ff:ff_light_ray_small ${cx} ${cy} ${cz}`
    )
  }

  const id = `ff:lamp_${turnOn ? 'off' : 'on'}_${wood}`
  block.dimension.runCommand(
    `setblock ${x} ${y} ${z} ${id}["minecraft:cardinal_direction"="${dir}","ef:colors"=${color}]`
  )
  player.playSound(`ff:lamp_${turnOn ? 'on' : 'off'}`)

  for (const neighbor of [block.above(1), block.below(1)]) {
    if (!neighbor) continue
    const nid = neighbor.typeId
    if ((nid.startsWith('ff:lamp_on_') || nid.startsWith('ff:lamp_off_')) 
        && getWoodType(neighbor) === wood) {
      toggleLamp(neighbor, turnOn, player, visited)
    }
  }
}

world.beforeEvents.worldInitialize.subscribe(initEvent => {
  const registry = initEvent.blockComponentRegistry

  registry.registerCustomComponent('ff:lamp_function_on', {
    onPlayerInteract: e => toggleLamp(e.block, true, e.player),
    onTick:           e => {
      const b = e.block
      if (!b.typeId.startsWith('ff:lamp_off_')) return
      const wood = getWoodType(b)
      if (!wood) return

      const { x, y, z } = b.location
      const cx = x + 0.5, cy = y, cz = z + 0.5
      const topBit    = b.permutation.getState("ff:top_bit")
      const bottomBit = b.permutation.getState("ff:bottom_bit")

      if (topBit && bottomBit) {
        b.dimension.runCommand(
          `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
        )
        return
      }
      if (!topBit) {
        b.dimension.spawnParticle(
          "ff:ff_light_ray_small",
          { x: cx, y: cy, z: cz }
        )
      }
    }
  })

  registry.registerCustomComponent('ff:lamp_function_off', {
    onPlayerInteract: e => toggleLamp(e.block, false, e.player),
    onTick:           e => {
      const b = e.block
      if (!b.typeId.startsWith('ff:lamp_on_')) return
      const wood = getWoodType(b)
      if (!wood) return

      const { x, y, z } = b.location
      const cx = x + 0.5, cy = y, cz = z + 0.5
      const topBit    = b.permutation.getState("ff:top_bit")
      const bottomBit = b.permutation.getState("ff:bottom_bit")

      if (topBit && bottomBit) {
        b.dimension.runCommand(
          `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
        )
      }
    }
  })

  const prevStates = new Map()
  registry.registerCustomComponent('ff:lamp_cx', {
    onTick: e => {
      const b = e.block
      const { x, y, z } = b.location
      const key = `${x},${y},${z}`

      const above  = b.above(1)
      const below  = b.below(1)
      const newTop    = !!above?.hasTag('ff:lamp_cx')
      const newBottom = !!below?.hasTag('ff:lamp_cx')

      const prev = prevStates.get(key) || { top: false, bottom: false }

      if ((newTop && !newBottom || (newTop && newBottom)) &&
          (newTop !== prev.top || newBottom !== prev.bottom)) {
        const cx = x + 0.5, cy = y, cz = z + 0.5
        b.dimension.runCommand(
          `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
        )
      }

      prevStates.set(key, { top: newTop, bottom: newBottom })

      b.setPermutation(
        b.permutation
         .withState("ff:top_bit",    newTop)
         .withState("ff:bottom_bit", newBottom)
      )
    }
  })
})

world.afterEvents.playerPlaceBlock.subscribe(data => {
  const b      = data.block
  const player = data.player
  if (!b.typeId.startsWith('ff:lamp_on_') && !b.typeId.startsWith('ff:lamp_off_')) return

  const turnOn = b.typeId.startsWith('ff:lamp_off_')

  if (turnOn) {
    const below = b.below(1)
    if (below && (below.typeId.startsWith('ff:lamp_on_') || below.typeId.startsWith('ff:lamp_off_'))) {
      const { x, y, z } = below.location
      below.dimension.runCommand(
        `execute positioned ${x + 0.5} ${y} ${z + 0.5} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
      )
    }
  }

  toggleLamp(b, turnOn, player)
})

world.afterEvents.playerBreakBlock.subscribe(data => {
  const old = data.block
  const player = data.player
  const { x, y, z } = old.location
  const cx = x + 0.5, cy = y, cz = z + 0.5

  old.dimension.runCommand(
    `execute positioned ${cx} ${cy} ${cz} run event entity @e[type=ff:ff_light_ray_small,r=0.5] destroy`
  )

  const below = old.below(1)
  if (below && (below.typeId.startsWith('ff:lamp_off_') || below.typeId.startsWith('ff:lamp_on_'))) {
    const newOn = below.typeId.startsWith('ff:lamp_off_')
    toggleLamp(below, newOn, player)
  }
})
